@@ -61,6 +61,9 @@ class LiveSurveyManager(models.Manager):
             models.Q(ends_at__gt=now))
 
 
+FORMAT_CHOICES = ('json', 'csv', 'xml', 'html',)
+
+
 class Survey(models.Model):
     title = models.CharField(max_length=80)
     slug = models.SlugField(unique=True)
@@ -116,7 +119,7 @@ class Survey(models.Model):
 
     def to_jsondata(self):
         kwargs = {'slug': self.slug}
-        submit_url = reverse('survey_detail', kwargs=kwargs)
+        submit_url = reverse('embeded_survey_questions', kwargs=kwargs)
         report_url = reverse('survey_default_report_page_1', kwargs=kwargs)
         questions = self.questions.order_by("order")
         return dict(title=self.title,
@@ -147,12 +150,26 @@ class Survey(models.Model):
             return self.starts_at <= now < self.ends_at
         return self.starts_at <= now
 
+    @property
+    def is_live(self):
+        now = datetime.datetime.now()
+        return all([
+            self.is_published,
+            self.starts_at <= now,
+            any([self.archive_policy != ARCHIVE_POLICY_CHOICES.NEVER,
+                not self.ends_at or self.ends_at < now])])
+
     def get_public_fields(self, fieldnames=None):
-        if not "_public_fields" in self.__dict__:
-            questions = self.questions.filter(answer_is_public=True)
+        if fieldnames:
+            return self.get_fields(fieldnames)
+        return [f for f in self.get_fields() if f.answer_is_public]
+
+    def get_fields(self, fieldnames=None):
+        if not "_fields" in self.__dict__:
+            questions = self.questions.all()
             questions = questions.select_related("survey")
-            self.__dict__["_public_fields"] = list(questions.order_by("order"))
-        fields = self.__dict__["_public_fields"]
+            self.__dict__["_fields"] = list(questions.order_by("order"))
+        fields = self.__dict__["_fields"]
         if fieldnames:
             return [f for f in fields if f.fieldname in fieldnames]
         return fields
@@ -216,6 +233,20 @@ class Survey(models.Model):
     def get_absolute_url(self):
         return ('survey_detail', (), {'slug': self.slug})
 
+    def get_download_url(self, format):
+        url = reverse('submissions_by_format', kwargs={"format": format})
+        return url + "?survey=" + self.slug
+
+    def get_download_tag(self, format):
+        a = '<a target="_blank" href="%s">%s</a>'
+        return a % (self.get_download_url(format), format,)
+
+    def get_download_tags(self, delimiter=", "):
+        downloads = []
+        for format in sorted(FORMAT_CHOICES):
+            downloads.append(self.get_download_tag(format))
+        return delimiter.join(downloads)
+
     objects = models.Manager()
     live = LiveSurveyManager()
 
@@ -264,7 +295,9 @@ class Question(models.Model):
         blank=True)
     required = models.BooleanField(
         default=False,
-        help_text=_("Unsafe to change on live surveys."))
+        help_text=_("Unsafe to change on live surveys. Radio button list and "
+                    "drop down list questions will have a blank option if "
+                    "they aren't required."))
     if PositionField:
         order = PositionField(collection=('survey',))
     else:
@@ -313,6 +346,12 @@ class Question(models.Model):
                     cms_id=self.id,
                     help_text=self.help_text)
 
+    @property
+    def public_answers(self):
+        if self.answer_is_public:
+            return self.answer_set.filter(submission__is_public=True)
+        return self.answer_set.none()
+
     class Meta:
         ordering = ('order',)
         unique_together = ('fieldname', 'survey')
@@ -530,7 +569,7 @@ class AggregateResultCount:
     """ This helper class makes it easier to write templates that display
     pie charts. """
     def __init__(self, survey, field, request_data):
-        self.answer_set = field.answer_set.values(field.value_column)
+        self.answer_set = field.public_answers.values(field.value_column)
         self.answer_set = self.answer_set.annotate(count=Count("id"))
         self.answer_set = extra_from_filters(self.answer_set,
                                              "submission_id",
@@ -592,7 +631,10 @@ class AggregateResult2Axis(object):
                 ") AS y_value FROM crowdsourcing_answer AS y_axis ",
                 "JOIN crowdsourcing_answer AS x_axis "
                 "ON y_axis.submission_id = x_axis.submission_id ",
-                "WHERE y_axis.question_id = %s AND ",
+                "JOIN crowdsourcing_submission AS submission ",
+                "ON submission.id = y_axis.submission_id ",
+                "WHERE submission.is_public = true AND ",
+                "y_axis.question_id = %s AND ",
                 y_axis_column,
                 " IS NOT NULL AND x_axis.question_id = %s"]
             y = "y_axis.submission_id"
@@ -605,7 +647,9 @@ class AggregateResult2Axis(object):
             query.append(x_value_column)
             cursor = connection.cursor()
             cursor.execute("".join(query), params)
+            found_any = False
             for x_value, y_value in cursor.fetchall():
+                found_any = True
                 if isinstance(y_value, Decimal):
                     y_value = round(y_value, 2)
                 answer_value = answer_value_lookup.get(x_value)
@@ -615,6 +659,8 @@ class AggregateResult2Axis(object):
         if x_axis.is_numeric:
             key = x_axis.fieldname
             self.answer_values.sort(lambda x, y: x[key] - y[key])
+        if not found_any:
+            self.answer_values = []
         self.yahoo_answer_string = json.dumps(self.answer_values)
 
 
@@ -645,25 +691,28 @@ class Submission(models.Model):
     featured = models.BooleanField(default=False)
 
     # for moderation
-    is_public = models.BooleanField(default=True)
+    is_public = models.BooleanField(
+        default=True,
+        help_text=_("Crowdsourcing only displays public submissions. The "
+                    "'Moderate submissions' checkbox of the survey determines "
+                    "the default value of this field."))
 
     class Meta:
         ordering = ('-submitted_at',)
 
-    def to_jsondata(self, answer_lookup=None):
+    def to_jsondata(self, answer_lookup=None, include_private_questions=False):
         def to_json(v):
             if isinstance(v, ImageFieldFile):
                 return v.url if v else ''
             return v
-        if answer_lookup:
-            answers = answer_lookup[self.pk]
-        else:
-            answers = self.answer_set.filter(question__answer_is_public=True)
+        if not answer_lookup:
+            answer_lookup = get_all_answers([self], include_private_questions)
         return_value = dict(data=dict((a.question.fieldname, to_json(a.value))
-                                      for a in answers),
+                                      for a in answer_lookup.get(self.pk, [])),
                             survey=self.survey.slug,
                             submitted_at=self.submitted_at,
-                            featured=self.featured)
+                            featured=self.featured,
+                            is_public=self.is_public)
         if self.user:
             return_value["user"] = self.user.username
         return return_value
@@ -741,9 +790,17 @@ class Answer(models.Model):
         ordering = ('question',)
 
     def save(self, **kwargs):
-        super(Answer, self).save(**kwargs)
         # or should this be in a signal?  Or build in an option
         # to manage asynchronously? @TBD
+        if local_settings.SYNCHRONOUS_FLICKR_UPLOAD:
+            self._sync_self_to_flickr()
+        super(Answer, self).save(**kwargs)
+
+    def __unicode__(self):
+        return unicode(self.question)
+
+    def _sync_self_to_flickr(self):
+        """ Does not save. You must save after syncing. """
         if sync_to_flickr:
             survey = self.question.survey
             if survey.flickr_group_id:
@@ -753,8 +810,17 @@ class Answer(models.Model):
                     message = "error in syncing to flickr: %s" % str(ex)
                     logging.exception(message)
 
-    def __unicode__(self):
-        return unicode(self.question)
+    @classmethod
+    def sync_to_flickr(cls):
+        if sync_to_flickr:
+            answers = cls.objects.filter(
+                image_answer__gt='',
+                flickr_id='',
+                question__survey__flickr_group_id__gt='')
+            answers = answers.select_related("question__survey")
+            for answer in answers:
+                answer._sync_self_to_flickr()
+                answer.save()
 
 
 class SurveyReport(models.Model):
@@ -765,8 +831,14 @@ class SurveyReport(models.Model):
     and an annotation.  It also has article-like fields of its own.
     """
     survey = models.ForeignKey(Survey)
-    title = models.CharField(max_length=50)
-    slug = models.CharField(max_length=50)
+    title = models.CharField(
+        max_length=50,
+        blank=True,
+        help_text=_("You may leave this field blank. Crowdsourcing will use "
+                    "the survey title as a default."))
+    slug = models.CharField(
+        max_length=50,
+        help_text=_("The default is the description of the survey."))
     # some text at the beginning
     summary = models.TextField(blank=True)
     # As crowdsourcing doesn't implement rating because we want to let you use
@@ -819,11 +891,18 @@ class SurveyReport(models.Model):
         unique_together = (('survey', 'slug'),)
         ordering = ('title',)
 
+    def get_title(self):
+        return self.title or self.survey.title
+
+    def get_summary(self):
+        return self.summary or self.survey.description or self.survey.tease
+
     def __unicode__(self):
-        return self.title
+        return self.get_title()
 
 
-SURVEY_DISPLAY_TYPE_CHOICES = ChoiceEnum('text pie map bar line slideshow')
+SURVEY_DISPLAY_TYPE_CHOICES = ChoiceEnum(
+    'text pie map bar line slideshow download')
 
 
 SURVEY_AGGREGATE_TYPE_CHOICES = ChoiceEnum('default sum count average')
@@ -836,8 +915,10 @@ class SurveyReportDisplay(models.Model):
         choices=SURVEY_DISPLAY_TYPE_CHOICES)
     aggregate_type = models.PositiveIntegerField(
         choices=SURVEY_AGGREGATE_TYPE_CHOICES,
-        help_text=_("We only use this field if you chose a Pie, Bar, or Line "
-                    "Chart."),
+        help_text=_("We only use this field if you chose a Bar or Line Chart. "
+                    "How should we aggregate the y-axis? 'Average' is good "
+                    "for things like ratings, 'Sum' is good for totals, and "
+                    "'Count' is good for a show of hands."),
         default=SURVEY_AGGREGATE_TYPE_CHOICES.DEFAULT)
     fieldnames = models.TextField(
         blank=True,
@@ -939,9 +1020,12 @@ class SurveyReportDisplay(models.Model):
         return super(SurveyReportDisplay, self).__getattribute__(key)
 
 
-def get_all_answers(submission_list):
+def get_all_answers(submission_list, include_private_questions=False):
     ids = [submission.id for submission in submission_list]
     page_answers_list = Answer.objects.filter(submission__id__in=ids)
+    if not include_private_questions:
+        kwargs = dict(question__answer_is_public=True)
+        page_answers_list = page_answers_list.filter(**kwargs)
     page_answers_list = page_answers_list.select_related("question")
     page_answers = {}
     for answer in page_answers_list:
